/*
 * ClassificationC.cc - Simple implementation of a fitnessfunction.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 06-Aug-1998
 *
 */

#include "ClassificationC.h"



ClassificationC::ClassificationC
(
	ConfigurationA const & iConfiguration,
	StatisticsA & iStatistics,
	StopConditionA const & iStopCondition,
	ProblemA const & iProblem
): FitnessFunctionA(iConfiguration, iStatistics, iStopCondition, iProblem)
{
	ProblemA const * tmp = &Problem;
	DataBaseProblem = dynamic_cast<DataBaseProblemC const *>(tmp);

	if (DataBaseProblem == 0)
		throw new InternalErrorC("ClassificationC got a problem that is not a DataBaseProblemC or one of its children.");
} // ClassificationC


ClassificationC::~ClassificationC()
{
} // ~ClassificationC



void const
ClassificationC::Calculate
(
	FitnessType & iFitnessValue,
	DataA const & iData
) const
{
	DataA const * tmp = &iData;
	TreeDataC * TreeData = const_cast<TreeDataC *>(dynamic_cast<TreeDataC const *>(tmp));
	if (TreeData == 0)
	{
		 throw new InternalErrorC("Got an incompatible data structure in ClassificationC::Calculate.");
	}

	ProblemA const * tmp2 = &Problem;
	DataBaseProblemC const * DataBaseProblem = dynamic_cast<DataBaseProblemC const *>(tmp2);
	if (TreeData == 0)
	{
		 throw new InternalErrorC("Got an incompatible problem in ClassificationC::Calculate.");
	}


	iFitnessValue = 0;
	unsigned int j = 0;

	for (DataBaseProblemC::ConstIteratorC i = DataBaseProblem->CreateConstIterator(); !i.IsEnd(); ++i)
	{
		if (TreeData->Tree.GetRoot().GetElement().Evaluate(TreeData->Tree.GetRoot(), *i))
		{
			++iFitnessValue;
			cerr << j++ << " ";
		}
	}

	cerr << "Fitness: " << iFitnessValue << endl;

	Statistics.RegisterEvaluation();

	StopCondition.RequestStop();
	//StopCondition.FoundSolution();

} // Calculate




// Eof ClassificationC.cc
