/*
 * PoolA.h - Abstract base class defining the interface of a population pool.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 24-Jun-1998
 *
 */

#ifndef PoolA_h
#define PoolA_h 1

#include "Core/GenotypeA.h"

class PoolIteratorA;
class PoolConstIteratorA;

/** Placeholder for a genotype. With the added feature of being able to hold a next and previous place.
 *
 * @memo Place inside the pool
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class PoolElementA
{
	public:
		PoolElementA
		(
			GenotypeA & iGenotype
		);

		virtual ~PoolElementA()
		{
		};
	
		virtual PoolElementA & GetNextPoolElement() const =0;
		virtual PoolElementA & GetPreviousPoolElement() const =0;

		virtual GenotypeA & GetGenotype() const
		{
			return *Genotype;
		};

		virtual void SetPreviousPoolElement
		(
			PoolElementA & iPoolElement
		) =0;
		
		virtual void SetNextPoolElement
		(
			PoolElementA & iPoolElement
		) =0;
		
	protected:

	private:

		GenotypeA * Genotype;

}; // PoolElementA



/** The pool is a container for holding the genotypes currently residing in the population. It's contents can be manipulated with the aid of iterators. There is not much to it then this, except that the pool keeps track of its current size. Because of the obvious names of all the members, no extra information is given for the seperate memberfunctions.
 *
 * @memo The pool
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class PoolA
{
	public:
	
		PoolA();

		virtual ~PoolA();

		virtual unsigned int GetSize() const =0;

		virtual void Add
		(
			GenotypeA & iGenotype
		) =0;

		virtual void Delete
		(
			PoolElementA & iPoolElement
		) =0;

		virtual PoolIteratorA* CreatePoolIterator() =0;
		virtual PoolIteratorA* CreatePoolReverseIterator() =0;
		virtual PoolConstIteratorA* CreatePoolConstIterator() const =0;
		virtual PoolConstIteratorA* CreatePoolReverseConstIterator() const =0;

		virtual PoolElementA & GetThePool() const =0;
		virtual PoolElementA & GetPoolEnd() const =0;

		friend ostream & operator<<
		(
			ostream & os,
			PoolA const & iPool
		)
		{
			iPool.PrintToStream(os);

			return os;
		};

	protected:

		virtual void PrintToStream
		(
			ostream & os
		) const =0;
		
		unsigned int Size;

	private:

}; // PoolA

#endif

// Eof PoolA.h
