/*
 * PoolC.h - Simple population pool, represented by a list.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 07-Jun-1998
 *
 */

#ifndef PoolC_h
#define PoolC_h 1

#include "Core/PoolA.h"

#include "Core/PoolIteratorC.h"
#include "Core/PoolReverseIteratorC.h"

//TODO: Move PoolElement into Pool class.


class PoolElementC: public PoolElementA
{
	public:

		PoolElementC
		(
			GenotypeA & iGenotype,
			PoolElementA & iPreviousPoolElement,
			PoolElementA & iNextPoolElement
		);

		virtual ~PoolElementC();

		virtual PoolElementA & GetNextPoolElement() const
		{
			return *NextPoolElement;
		};

		virtual PoolElementA & GetPreviousPoolElement() const
		{
			return *PreviousPoolElement;
		};

	protected:

		virtual void SetPreviousPoolElement
		(
			PoolElementA & iPoolElement
		);
		
		virtual void SetNextPoolElement
		(
			PoolElementA & iPoolElement
		);

	private:
		PoolElementA * NextPoolElement;
		PoolElementA * PreviousPoolElement;
}; // PoolElementC

class PoolC: public PoolA
{
	public:
	
		PoolC();

		/*PoolC
		(
			PoolC const & iPool
		);*/

		virtual ~PoolC();

		unsigned int GetSize() const
		{
			return Size;
		};

		virtual void Add
		(
			GenotypeA & iGenotype
		);

		virtual void Delete
		(
			PoolElementA & iPoolElement
		);

		virtual PoolIteratorA* CreatePoolIterator()
		{
			return new PoolIteratorC(*this);
		};

		virtual PoolIteratorA* CreatePoolReverseIterator()
		{
			return new PoolReverseIteratorC(*this);
		};

		virtual PoolConstIteratorA* CreatePoolConstIterator() const
		{
			return new PoolConstIteratorC(*this);
		};

		virtual PoolConstIteratorA* CreatePoolReverseConstIterator() const
		{
			return new PoolReverseConstIteratorC(*this);
		};

		virtual PoolElementA & GetThePool() const
		{
			return *ThePool;
		};

		virtual PoolElementA & GetPoolEnd() const
		{
			return *PoolEnd;
		};

		friend class PoolIteratorC;
		friend class PoolReverseIteratorC;
		friend class PoolConstIteratorC;
		friend class PoolReverseConstIteratorC;

	protected:

		virtual void PrintToStream
		(
			ostream & os
		) const;

		PoolElementA * ThePool;
		PoolElementA * PoolEnd;

	private:

}; // PoolC

#endif

// Eof PoolC.h
