/*
 * ExceptionA.h - A base class for exceptions.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 24-Jun-1998
 *
 */

#ifndef ExceptionA_h
#define ExceptionA_h 1

#include "Common/StringC.h"

/** Base class for all kinds of errors and warnings within the library. These exceptions will be created and then trown by objects, they should be catched, handled and neatly disposed of. \begin{description} \item[]{\tt catch(ExceptionA* Exception)} \item[] #{# \begin{description} \item[]{\tt Exception->Handle();} \item[]{\tt delete Exception;} \end{description} \item[] #}# \end{description}
 *
 * @memo An exception
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class ExceptionA 
{
	public:

		ExceptionA
		(
			char const * iChar
		);

		virtual ~ExceptionA () =0;


		/** This function should printout a newline and its name appended with a colon, call {\tt PrintMessage()} and then do what is should do. For example in the case of a fatal error the following will be the case:
		 * \begin{description} \item[]{\tt void FatalError::Handle()} \item[] #{# \begin{description} \item[]{\tt cerr << endl << "FatalError: ";} \item[]{\tt PrintMessage();} \item[]{\tt exit(1);} \end{description} \item[] #}#  \end{description}
		 *
		 * @memo Handling an exception
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		virtual void Handle() const =0;

	protected:

		virtual void PrintMessage() const;

	private:

		StringC * Message;

}; // ExceptionA

#endif

// Eof ExceptionA.h
