/*
 * PoolReverseIteratorC.cc - Simple reverse iterator to work with the PoolC class which implements a population as a simple list.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 10-Jun-1998
 *
 */

#include "Core/PoolReverseIteratorC.h"



PoolReverseIteratorC::PoolReverseIteratorC
(
	PoolA & iPool
)
{
	Cursor = 0;

	Pool = &iPool;

} // PoolReverseIteratorC



void 
PoolReverseIteratorC::First()
{
	Cursor = &(Pool->GetPoolEnd());
};



void 
PoolReverseIteratorC::Next()
{
	if (Cursor != 0)
		Cursor = &(Cursor->GetPreviousPoolElement());
	else
		throw new InternalErrorC("Tried to walk beyond the first individual in the pool.");
};



void 
PoolReverseIteratorC::Delete()
{
	if (Cursor != 0)
	{
		PoolElementA * Tmp = Cursor;
		Next();
		Pool->Delete(*Tmp);
	}
	else
		throw new InternalErrorC("Tried to free a nonexisting item in the pool using a reverseiterator");
};



GenotypeA & 
PoolReverseIteratorC::CurrentItem() const
{
	if (Cursor != 0)
		return Cursor->GetGenotype();
	else
		throw new InternalErrorC("Requested an item from the pool with a reverseiterator which is not pointing to something.");
};



bool 
PoolReverseIteratorC::IsDone() const
{
	return (Cursor == 0);
};


//TODO: Finish copy ctors for PoolIterator.
/*PoolReverseIteratorC::PoolReverseIteratorC
(
	PoolReverseIteratorC const & iPoolIterator
)
{

} // PoolReverseIteratorC


PoolReverseIteratorC const &
PoolReverseIteratorC::PoolReverseIteratorC
(
	PoolReverseIteratorC const & iPoolIterator
)
{

} // PoolReverseIteratorC
*/



PoolReverseIteratorC::~PoolReverseIteratorC()
{
} // ~PoolReverseIteratorC



//------------------------------------------------------



PoolReverseConstIteratorC::PoolReverseConstIteratorC
(
	PoolA const & iPool
)
{
	Cursor = 0;

	Pool = &iPool;

} // PoolReverseConstIteratorC



void 
PoolReverseConstIteratorC::First()
{
	Cursor = &(Pool->GetPoolEnd());
};



void 
PoolReverseConstIteratorC::Next()
{
	if (Cursor != 0)
		Cursor = &(Cursor->GetPreviousPoolElement());
	else
		throw new InternalErrorC("Tried to walk beyond the first individual in the pool.");
};



GenotypeA const & 
PoolReverseConstIteratorC::CurrentItem() const
{
	if (Cursor != 0)
		return Cursor->GetGenotype();
	else
		throw new InternalErrorC("Requested an item from the pool with a reverseiterator which is not pointing to something.");
};



bool 
PoolReverseConstIteratorC::IsDone() const
{
	return (Cursor == 0);
};


//TODO: Finish copy ctors for PoolIterator.
/*PoolReverseConstIteratorC::PoolReverseConstIteratorC
(
	PoolReverseConstIteratorC const & iPoolIterator
)
{

} // PoolReverseConstIteratorC


PoolReverseConstIteratorC const &
PoolReverseConstIteratorC::PoolReverseConstIteratorC
(
	PoolReverseConstIteratorC const & iPoolIterator
)
{

} // PoolReverseConstIteratorC
*/



PoolReverseConstIteratorC::~PoolReverseConstIteratorC()
{
} // ~PoolReverseConstIteratorC




// Eof PoolReverseIteratorC.cc
