/*
 * ProduceOperatorC.cc - Simple implementation of a produce operator: one point crossover.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 07-Aug-1998
 *
 */

#include "Reproduction/ProduceOperatorC.h"
#include "Common/ArrayC.tpl"



ProduceOperatorC::ProduceOperatorC
(
	ConfigurationA const & iConfiguration,
	CoreFactoryA const & iCoreFactory,
	ProblemFactoryA const & iProblemFactory,
	ProblemA const &iProblem,
	RandomA const & iRandom
): ProduceOperatorA(iConfiguration, iCoreFactory, iProblemFactory, iProblem, iRandom)
{
	NumberOfGenotypesPerOperation = (Configuration)("SelectParents", "GenotypesPerOperation");

} // ProduceOperatorC



ProduceOperatorC::~ProduceOperatorC()
{
};



ArrayC<GenotypeA*>*
ProduceOperatorC::operator()
(
	ArrayC<GenotypeA const *> const & iArray
)
{
	ArrayC<GenotypeA*> * Result = new ArrayC<GenotypeA*>(2, "List of children");

	GenotypeA * Child1 = CoreFactory.CreateGenotype(ProblemFactory);
	GenotypeA * Child2 = CoreFactory.CreateGenotype(ProblemFactory);
	(*Result)[0] = Child1;
	(*Result)[1] = Child2;

	unsigned int Cutpoint = (Random)(Child1->GetData().GetSize());

	DataA::DataConstIteratorA * Parent1Iterator = iArray[0]->GetData().CreateDataConstIterator();
	DataA::DataConstIteratorA * Parent2Iterator = iArray[1]->GetData().CreateDataConstIterator();

	DataA::DataIteratorA * Child1Iterator = Child1->GetData().CreateDataIterator();
	DataA::DataIteratorA * Child2Iterator = Child2->GetData().CreateDataIterator();

	Parent1Iterator->First();
	Parent2Iterator->First();
	Child1Iterator->First();
	Child2Iterator->First();
	unsigned int i = 0;

	while (!Child1Iterator->IsDone())
	{
		if (i < Cutpoint)
		{
			Child1Iterator->CurrentItem() = Parent1Iterator->CurrentItem();
			Child2Iterator->CurrentItem() = Parent2Iterator->CurrentItem();
		}
		else
		{
			Child1Iterator->CurrentItem() = Parent2Iterator->CurrentItem();
			Child2Iterator->CurrentItem() = Parent1Iterator->CurrentItem();
		}

		++i;
		Parent1Iterator->Next();
		Parent2Iterator->Next();
		Child1Iterator->Next();
		Child2Iterator->Next();
	}

#ifdef DEBUG
	cerr << "+ " << (*Child1) << endl;
	cerr << "  " << (*Child2) << endl;
#endif

	delete Parent1Iterator;
	delete Parent2Iterator;
	delete Child1Iterator;
	delete Child2Iterator;

	return Result;

} // operator()



// Eof ProduceOperatorC.cc
