/*
 * ReproductionFactoryC.cc - Class for instantiating objects within the Reproduction package.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 07-Aug-1998
 *
 */

#include "Reproduction/ReproductionFactoryC.h"



ReproductionA* 
ReproductionFactoryC::CreateReproduction
(
	CoreFactoryA const & iCoreFactory,
	ProblemFactoryA const & iProblemFactory,
	ProblemA const & iProblem,
	StopConditionA const & iStopCondition
) const
{
	return new ReproductionC(Configuration, iCoreFactory, iProblemFactory, *this, iStopCondition, iProblem, Random);

} // CreateReproduction



SelectSurvivorsA* 
ReproductionFactoryC::CreateSelectSurvivors() const
{
	return new SelectSurvivorsC(Configuration);

} // CreateSelectSurvivors



SelectParentsA* 
ReproductionFactoryC::CreateSelectParents() const
{
	return new SelectParentsC(Configuration, Random);

} // CreateSelectParents



ChangeOperatorA* 
ReproductionFactoryC::CreateChangeOperator
(
	ProblemFactoryA const & iProblemFactory,
	ProblemA const & iProblem
) const
{
	return new ChangeOperatorC(Configuration, iProblemFactory, iProblem, Random);

} // CreateChangeOperator



ProduceOperatorA* 
ReproductionFactoryC::CreateProduceOperator
(
	CoreFactoryA const & iCoreFactory,
	ProblemFactoryA const & iProblemFactory,
	ProblemA const & iProblem
) const
{
	return new ProduceOperatorC(Configuration, iCoreFactory, iProblemFactory, iProblem,  Random);

} // CreateProduceOperator



// Eof ReproductionFactoryC.cc
