/*
 * StatisticsC.cc - A basic statistics class: SR, AES and MF.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 17-Jun-1998
 *
 */

#include "StatisticsC.h"



StatisticsC::StatisticsC
(
	ConfigurationA const & iConfiguration
): StatisticsA(iConfiguration)
{
} // StatisticsC



StatisticsC::~StatisticsC()
{

} // ~StatisticsC



void
StatisticsC::ResetRunStatistics()
{
	StatisticsA::ResetRunStatistics();

} // ResetRunStatistics



void
StatisticsC::CollectBasicRunStatistics
(
	bool iIsSucces,
	FitnessType const & iBestFitness
)
{
	++NumberOfRuns;

	TotalFitness += iBestFitness;

	//REM: cout << iBestFitness << " " << NumberOfEvaluations << endl;

	if (iIsSucces)
	{
		TotalNumberOfEvaluations += NumberOfEvaluations;
		TotalSuccess++;
	}

	NumberOfEvaluations = 0;

} // CollectBasicRunStatistics



void
StatisticsC::PrintToStream
(
	ostream & iOstream
) const
{
	iOstream
		<< setw(iOstream.width()) 
		<< NumberOfRuns
		<< " "
		<< (double) TotalFitness / (double) NumberOfRuns
		<< " "
		<< (double) TotalSuccess / (double) NumberOfRuns
		<< " "
	;

	if (TotalSuccess > 0)
		iOstream << (double) ((double) TotalNumberOfEvaluations / (double) TotalSuccess);
	else
		iOstream << "oo";

} // PrintToStream



// Eof StatisticsC.cc
